#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/14 23:41
@File: test_log.py
@IDE: pycharm
@Description:
    测试日志模块
"""
from swankit.log import SwanLabSharedLog
import nanoid


class TestSwanKitLog:
    """
    测试日志模块
    """

    def test_enable_default(self, capsys):
        """
        测试开启日志
        """
        levels = ["debug", "info", "warning", "error", "critical"]
        for level in levels:
            name = nanoid.generate()
            text = nanoid.generate()
            t = SwanLabSharedLog(name, level=level)
            for le in levels:
                getattr(t, le)(text)
                out, err = capsys.readouterr()
                if levels.index(le) >= levels.index(level):
                    assert text in out
                    assert name in out
                    assert err == ""
                else:
                    assert out == ""
                    assert err == ""

    def test_disable(self, capsys):
        """
        测试关闭日志
        """
        levels = ["debug", "info", "warning", "error", "critical"]
        for level in levels:
            name = nanoid.generate()
            text = nanoid.generate()
            t = SwanLabSharedLog(name, level=level)
            t.disable_log()
            for le in levels:
                getattr(t, le)(text)
                out, err = capsys.readouterr()
                assert out == ""
                assert err == ""

    def test_set_level(self, capsys):
        """
        测试设置日志等级
        """
        levels = ["debug", "info", "warning", "error", "critical"]
        for le in levels:
            name = nanoid.generate()
            text = nanoid.generate()
            t = SwanLabSharedLog(name, level="debug")
            t.level = le
            getattr(t, le)(text)
            out, err = capsys.readouterr()
            assert text in out
            assert name in out
            assert err == ""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/5 16:29
@File: __init__.py
@IDE: pycharm
@Description:
    与回调函数通信时的模型
"""
from .key import MediaBuffer, MetricInfo, ColumnInfo, MetricErrorInfo, ColumnClass, SectionType, ColumnConfig, YRange
from .error import OperateErrorInfo
from .runtime import RuntimeInfo

__all__ = [
    "ColumnInfo",
    "MediaBuffer",
    "MetricInfo",
    "MetricErrorInfo",
    "OperateErrorInfo",
    "RuntimeInfo",
    "ColumnClass",
    "SectionType",
    "ColumnConfig",
    "YRange",
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/14 23:07
@File: env.py
@IDE: pycharm
@Description:
    规定可以被复用的、全局的环境变量，原则上，这里的环境变量和工具应该影响到所有基于 SwanLab-Toolkit 开发的项目
"""
import os
import sys
from enum import Enum
from platform import platform
from typing import List
import datetime


class SwanLabMode(Enum):
    """
    swanlab的解析模式，枚举类
    """

    DISABLED = "disabled"
    CLOUD = "cloud"
    CLOUD_ONLY = "cloud-only"
    LOCAL = "local"

    @classmethod
    def list(cls) -> List[str]:
        """
        获取所有的枚举值
        :return: 所有的枚举值
        """
        return [item.value for item in cls]


class SwanLabSharedEnv(Enum):
    """
    环境变量Key，枚举类，这些环境变量是swanlab全局共用的环境变量
    """

    SWANLAB_FOLDER = "SWANLAB_SAVE_DIR"
    """
    swanlab全局文件夹保存的路径，默认为用户主目录下的.swanlab文件夹
    """
    SWANLOG_FOLDER = "SWANLAB_LOG_DIR"
    """
    swanlab解析日志文件保存的路径，默认为当前运行目录的swanlog文件夹
    """
    SWANLAB_MODE = "SWANLAB_MODE"
    """
    swanlab的解析模式，涉及操作员注册的回调，目前有三种：local、cloud、disabled，默认为cloud
    大小写不敏感
    """

    @classmethod
    def list(cls) -> List[str]:
        """
        获取所有的枚举值
        :return: 所有的枚举值
        """
        return [item.value for item in cls]


# ---------------------------------- 获取环境变量/配置的值 ----------------------------------


def is_windows() -> bool:
    """判断当前操作系统是否是windows还是类unix系统，主要是路径分隔上的差别
    此外的系统会报错为 UnKnownSystemError
    :raise OSError: 未知系统错误，此时swanlab运行在未知系统上，这个系统不是windows或者类unix系统
    :return: True表示是windows系统，False表示是类unix系统
    """
    if sys.platform.startswith("win"):
        return True
    elif sys.platform.startswith("linux") or sys.platform.startswith("darwin"):
        return False
    raise OSError("Unknown system, not windows or unix-like system")


def is_macos() -> bool:
    """判断当前操作系统是否是macos
    :return: True表示是macos系统，False表示不是macos系统
    """
    return "mac" in platform().lower()


def create_time() -> str:
    """获取当前时间(UTC时区)"""
    return datetime.datetime.now(datetime.timezone.utc).isoformat()


def get_save_dir() -> str:
    """
    获取存放swanlab全局文件的文件夹路径，如果不存在就创建
    此函数对应为SWANLAB_SAVE_FOLDER全局变量，如果没有设置，默认为用户主目录下的.swanlab文件夹
    执行此函数时，如果文件夹不存在，自动创建，但是出于安全考虑，不会自动创建父文件夹
    :raises
        :raise FileNotFoundError: folder的父目录不存在
        :raise NotADirectoryError: folder不是一个文件夹
    :return: swanlab全局文件夹保存的路径，返回处理后的绝对路径
    """
    folder = os.getenv(SwanLabSharedEnv.SWANLAB_FOLDER.value)
    if folder is None:
        folder = os.path.join(os.path.expanduser("~"), ".swanlab")
    folder = os.path.abspath(folder)
    if not os.path.exists(os.path.dirname(folder)):
        raise FileNotFoundError(f"{os.path.dirname(folder)} not found")
    if not os.path.exists(folder):
        # 只创建当前文件夹，不创建父文件夹
        os.mkdir(folder)
    if not os.path.isdir(folder):
        raise NotADirectoryError(f"{folder} is not a directory")
    return folder


def get_swanlog_dir() -> str:
    """
    获取存放swanlog日志文件的文件夹路径
    此函数对应为SWANLAB_LOG_FOLDER全局变量，如果没有设置，默认为当前运行目录下的swanlog文件夹
    需要注意，此函数并不会保证文件夹的存在，但是会检查父文件夹是否存在以及folder是否是一个文件夹
    :raises
        :raise FileNotFoundError: folder的父目录不存在
        :raise NotADirectoryError: folder不是一个文件夹
    :return: swanlog日志文件保存的路径，返回处理后的绝对路径
    """
    folder = os.getenv(SwanLabSharedEnv.SWANLOG_FOLDER.value)
    if folder is None:
        folder = os.path.join(os.getcwd(), "swanlog")
    folder = os.path.abspath(folder)
    if not os.path.exists(os.path.dirname(folder)):
        raise FileNotFoundError(f"{os.path.dirname(folder)} not found")
    if not os.path.exists(folder):
        return folder
    if not os.path.isdir(folder):
        raise NotADirectoryError(f"{folder} is not a directory")
    return folder


def get_mode() -> str:
    """
    获取当前的swanlab解析模式，如果没有设置，默认为cloud
    :raise ValueError: 未知的swanlab模式
    :return: swanlab的解析模式
    """
    mode = os.getenv(SwanLabSharedEnv.SWANLAB_MODE.value)
    if mode is None:
        mode = SwanLabMode.CLOUD.value
    mode = mode.lower()
    if mode not in SwanLabMode.list():
        raise ValueError(f"Unknown swanlab mode: {mode}")
    return mode



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/15 13:24
@File: settings.py
@IDE: pycharm
@Description:
    swankit 为 swanlab 定制的配置类
"""
import os
from typing import Tuple


class LazySettings:
    """
    运行时动态设置的信息，只能设置一次，他们并不在一开始就被赋予意义，如果在设置前访问，返回None
    """

    def __init__(self):
        self.__exp_name = None
        self.__exp_colors = None
        self.__description = None

    @property
    def exp_name(self) -> str:
        """实验名称"""
        if self.__exp_name is None:
            raise ValueError("exp_name is None before set")
        return self.__exp_name

    @exp_name.setter
    def exp_name(self, exp_name: str) -> None:
        """实验名称"""
        if self.__exp_name is not None:
            raise ValueError("exp_name can only be set once")
        self.__exp_name = exp_name

    @property
    def exp_colors(self) -> Tuple[str, str]:
        """实验颜色"""
        return self.__exp_colors

    @exp_colors.setter
    def exp_colors(self, exp_colors: Tuple[str, str]) -> None:
        """实验颜色"""
        if self.__exp_colors is not None:
            raise ValueError("exp_colors can only be set once")
        self.__exp_colors = exp_colors

    @property
    def description(self) -> str:
        """实验描述"""
        return self.__description

    @description.setter
    def description(self, description: str) -> None:
        """实验描述"""
        if self.__description is not None:
            raise ValueError("description can only be set once")
        self.__description = description


class SwanLabSharedSettings(LazySettings):
    """
    SwanLabRun的配置信息，包括当前实验路径信息等

    涉及路径的属性都已经自动转换为绝对路径，并且文件夹路径在`should_save=True`时会自动创建

    "几乎"所有属性都为只读属性，只有在初始化时可以设置
    """

    def __init__(self, logdir: str, run_id: str, version: str, should_save: bool) -> None:
        """
        初始化
        :param logdir: 日志目录
        :param run_id: 实验运行id
        :param version: 实验版本
        :param should_save: 是否应该保存实验相关信息，如果保存，相关文件夹将自动创建（文件不会自动创建）
        """
        LazySettings.__init__(self)
        # ---------------------------------- 静态信息 ----------------------------------
        self.__should_save = should_save
        self.__run_id = run_id
        self.__version = version
        # ---------------------------------- 文件夹信息 ----------------------------------
        self.__root_dir = os.path.dirname(logdir)
        self.__swanlog_dir = logdir
        self.__run_dir = os.path.join(logdir, run_id)
        self.__console_dir = os.path.join(self.run_dir, "console")
        self.__log_dir = os.path.join(self.run_dir, "logs")
        self.__files_dir = os.path.join(self.run_dir, "files")
        self.__media_dir = os.path.join(self.run_dir, "media")
        # ---------------------------------- 文件信息 ----------------------------------
        self.__error_path = os.path.join(self.console_dir, "error.log")

    def mkdir(self, path: str) -> None:
        """创建目录
        为了保证安全性，不会递归创建
        """
        if not os.path.exists(path) and self.should_save:
            os.mkdir(path)

    # ---------------------------------- 静态属性 ----------------------------------

    @property
    def should_save(self):
        """
        是否应该保存实验信息
        """
        return self.__should_save

    @property
    def version(self) -> str:
        return self.__version

    @property
    def run_id(self) -> str:
        """实验运行id"""
        return self.__run_id

    # ---------------------------------- 文件夹属性 ----------------------------------

    @property
    def root_dir(self) -> str:
        """swanlog保存的根目录"""
        # 必然存在，不需要创建
        return self.__root_dir

    @property
    def swanlog_dir(self) -> str:
        """swanlog目录，存储所有实验的日志目录，也是runs.swanlab数据库的存储目录"""
        self.mkdir(self.__swanlog_dir)
        return self.__swanlog_dir

    @property
    def run_dir(self) -> str:
        """实验日志、信息文件夹路径"""
        self.mkdir(self.__run_dir)
        return self.__run_dir

    @property
    def log_dir(self) -> str:
        """记录用户日志文件夹路径"""
        self.mkdir(self.__log_dir)
        return self.__log_dir

    @property
    def console_dir(self) -> str:
        """记录终端日志路径"""
        self.mkdir(self.__console_dir)
        return self.__console_dir

    @property
    def media_dir(self) -> str:
        """静态资源路径"""
        self.mkdir(self.__media_dir)
        return self.__media_dir

    @property
    def files_dir(self) -> str:
        """实验配置信息路径"""
        self.mkdir(self.__files_dir)
        return self.__files_dir

    # ---------------------------------- 文件属性 ----------------------------------

    @property
    def error_path(self) -> str:
        """错误日志文件路径"""
        return self.__error_path



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/4/3 16:52
@File: conftest.py.py
@IDE: pycharm
@Description:
    配置pytest
"""
import pytest
from tutils import TEMP_DIR
from swankit.env import SwanLabSharedEnv
import shutil
import os


@pytest.fixture(scope="function", autouse=True)
def setup_before_each():
    # ---------------------------------- 在每一个函数执行前，删除临时文件夹 ----------------------------------
    if os.path.exists(TEMP_DIR):
        shutil.rmtree(TEMP_DIR)
    os.mkdir(TEMP_DIR)
    # ---------------------------------- 每个函数执行前，清空环境变量 ----------------------------------
    for key in SwanLabSharedEnv.list():
        if key in os.environ:
            del os.environ[key]
    yield



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/14 23:19
@File: log.py
@IDE: pycharm
@Description:
    日志模块，封装logging模块，提供swanlab标准日志记录功能
"""
from .utils import FONT
from typing import Literal, Union
import sys

# ---------------------------------- 日志打印等级 ----------------------------------
Levels = Union[Literal["debug", "info", "warning", "error", "critical"], str]
"""
SwanKitLog 预先定义好的日志等级字符串
"""
CRITICAL = 50
ERROR = 40
WARNING = 30
INFO = 20
DEBUG = 10


# ---------------------------------- 日志类 ----------------------------------


class SwanLabSharedLog:

    def __init__(self, name=__name__.lower(), level: Levels = "info", file=None):
        self.__file = file if file else sys.stdout
        self.__level: int = 0
        # 粗体
        self.__prefix_dict = {
            "debug": FONT.bold(FONT.grey(name)) + ":",
            "info": FONT.bold(FONT.blue(name)) + ":",
            "warning": FONT.bold(FONT.yellow(name)) + ":",
            "error": FONT.bold(FONT.red(name)) + ":",
            "critical": FONT.bold(FONT.bold(FONT.red(name))) + ":",
        }
        self.__levels_dict = {
            "debug": DEBUG,
            "info": INFO,
            "warning": WARNING,
            "error": ERROR,
            "critical": CRITICAL,
        }
        self.level = level
        self.__can_log = True

    @property
    def level(self):
        return self.__level

    @level.setter
    def level(self, level: Levels):
        """
        设置日志等级
        :param level: 日志等级，可选值为 debug, info, warning, error, critical，如果传入的值不在可选值中，则默认为 info
        """
        self.__level = self.__levels_dict.get(level.lower(), 20)

    def disable_log(self):
        """
        关闭日志输出，实例化时默认开启
        """
        self.__can_log = False

    def enable_log(self):
        """
        开启日志输出
        """
        self.__can_log = True

    def __print(self, log_level: str, *args, **kwargs):
        """
        打印日志
        """
        if not self.__can_log:
            return
        level = self.__levels_dict[log_level]
        if level < self.__level:
            return
        print(self.__prefix_dict[log_level], *args, **kwargs, file=self.__file)

    # 发送调试消息
    def debug(self, *args, **kwargs):
        return self.__print("debug", *args, **kwargs)

    # 发送通知
    def info(self, *args, **kwargs):
        return self.__print("info", *args, **kwargs)

    # 发生警告
    def warning(self, *args, **kwargs):
        return self.__print("warning", *args, **kwargs)

    # 发生错误
    def error(self, *args, **kwargs):
        return self.__print("error", *args, **kwargs)

    # 致命错误
    def critical(self, *args, **kwargs):
        return self.__print("critical", *args, **kwargs)
